/// subroutines for reading and writing hdf5part files
//#include "hdf5.h"
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include <assert.h>
#include "nbody.h"
// g++ junk2.cpp -l H5Part -l hdf5 -o junk2.out

void ckfile(char *filename);
#ifdef H5PART
#include "H5Part.h"


// remember if you have intention of restart option using
// last ouput then you need to make sure all simulation params
// are stored in file, 
// right now niter missing


// if timeStep == 0 then store the info in the simulation info struct
// as file attributes
// write all the data out for the timestep into the file
void write_h5part(char *filename, double* Pos, double *Vel, SIMinfo siminfo, int timeStep)
{
    H5PartFile *fileID;
    /* Open an HDF5 file for writing */
    fileID = H5PartOpenFile(filename,H5PART_WRITE);
//    printf("hdf hello2\n"); fflush(stdout);

    H5PartSetStep(fileID,timeStep);  // set timestep
    long long nparticles=siminfo.numBodies;
    H5PartSetNumParticles(fileID,nparticles); // set number of particles
 //   printf("w5 NB %d\n",nparticles); 

//  transfer data to double
    double *x,*y,*z,*px,*py,*pz,*m;
    x = new double[nparticles];
    y = new double[nparticles];
    z = new double[nparticles];
    px = new double[nparticles];
    py = new double[nparticles];
    pz = new double[nparticles];
    m = new double[nparticles];
    int i;
    for(i=0;i<nparticles;i++){
      x[i] = (double)Pos[4*i+0];  // offset must be checked 
      y[i] = (double)Pos[4*i+1];
      z[i] = (double)Pos[4*i+2];
      m[i] = (double)Pos[4*i+3];
      px[i] = (double)Vel[4*i+0];
      py[i] = (double)Vel[4*i+1];
      pz[i] = (double)Vel[4*i+2];
    }

   //write particle data
    H5PartWriteDatadouble64(fileID,"x",x);
    H5PartWriteDatadouble64(fileID,"y",y);
    H5PartWriteDatadouble64(fileID,"z",z);
    H5PartWriteDatadouble64(fileID,"m",m);
    H5PartWriteDatadouble64(fileID,"px",px);
    H5PartWriteDatadouble64(fileID,"py",py);
    H5PartWriteDatadouble64(fileID,"pz",pz);
   // clean up data arrays
    delete [] x;
    delete [] y;
    delete [] z;
    delete [] px;
    delete [] py;
    delete [] pz;
    delete [] m;

   // write some attributes to file if first step
    if (timeStep==0){
       double att1;
       int att2;
       att1 = (double)siminfo.softening;
       H5PartWriteFileAttrib(fileID,"softening",H5T_NATIVE_DOUBLE,&att1,1);
       att1 = (double)siminfo.damping;
       H5PartWriteFileAttrib(fileID,"damping",H5T_NATIVE_DOUBLE,&att1,1);
       att1 = (double)siminfo.deltaTime;
       H5PartWriteFileAttrib(fileID,"deltaTime",H5T_NATIVE_DOUBLE,&att1,1);
       att2 = (int)siminfo.niter;
       H5PartWriteFileAttrib(fileID,"niter",H5T_NATIVE_INT,&att2,1);
       att2 = (int)siminfo.ndt;
       H5PartWriteFileAttrib(fileID,"ndt",H5T_NATIVE_INT,&att2,1);
       att1 = (double)siminfo.time;
       H5PartWriteFileAttrib(fileID,"time",H5T_NATIVE_DOUBLE,&att1,1);
    }


    // close it
    H5PartCloseFile(fileID);
}

// return the siminfo struc in an H5part file 
void info_h5part(char *filename, SIMinfo *siminfo)
{
    ckfile(filename); // check if file exists
    H5PartFile *fileID;
    /* Open an HDF5 file for read */
    fileID = H5PartOpenFile(filename,H5PART_READ);
    
/*
    if (!H5PartFileIsValid(fileID)){
      printf("H5PartFile not found %s\n",filename);
      exit(0);
    }
*/

    H5PartSetStep(fileID,0);  	// set timestep to first one, index is 0
    long long nparticles=0;
    nparticles = H5PartGetNumParticles(fileID);
    siminfo->numBodies = nparticles;
    double att1;
    H5PartReadFileAttrib(fileID,"softening",&att1);
    siminfo->softening=att1;
    H5PartReadFileAttrib(fileID,"damping",&att1);
    siminfo->damping=att1;
    H5PartReadFileAttrib(fileID,"deltaTime",&att1);
    siminfo->deltaTime=att1;
    H5PartReadFileAttrib(fileID,"time",&att1);
    siminfo->time=att1;

    H5PartCloseFile(fileID);
}

// note figures out the numbers of particles
void read_h5part(char *filename, double *Pos, double *Vel, int timeStep)
{
    ckfile(filename); // check if file exists
    H5PartFile *fileID;
    /* Open an HDF5 file for read */
    fileID = H5PartOpenFile(filename,H5PART_READ);

/*
    if (!H5PartFileIsValid(fileID)){
      printf("H5PartFile not found %s\n",filename);
      exit(0);
    }
*/

    H5PartSetStep(fileID,timeStep); // set timestep 
    long long nparticles=0;
    nparticles = H5PartGetNumParticles(fileID);
    double *x,*y,*z,*px,*py,*pz,*m;
    x = new double[nparticles];
    y = new double[nparticles];
    z = new double[nparticles];
    px = new double[nparticles];
    py = new double[nparticles];
    pz = new double[nparticles];
    m = new double[nparticles];

// read in some stuff
    H5PartReadDatadouble64(fileID,"x",x);
    H5PartReadDatadouble64(fileID,"y",y);
    H5PartReadDatadouble64(fileID,"z",z);
    H5PartReadDatadouble64(fileID,"m",m);
    H5PartReadDatadouble64(fileID,"px",px);
    H5PartReadDatadouble64(fileID,"py",py);
    H5PartReadDatadouble64(fileID,"pz",pz);

// pack into arrays
    int i;
    for(i=0;i<nparticles;i++){
      Pos[4*i+0] = x[i];  // offset must be checked   
      Pos[4*i+1] = y[i];
      Pos[4*i+2] = z[i];
      Pos[4*i+3] = m[i];
      Vel[4*i+0] = px[i];
      Vel[4*i+1] = py[i];
      Vel[4*i+2] = pz[i];
    }

    H5PartCloseFile(fileID); // close file
   // clean up data arrays
    delete [] x;
    delete [] y;
    delete [] z;
    delete [] px;
    delete [] py;
    delete [] pz;
    delete [] m;

}
#endif 

// check that file exists
void ckfile(char *filename)
{
  FILE *fp;
  fp = fopen(filename,"r");
  if (fp == NULL){
     printf("%s file not found \n", filename);
     fclose(fp);
     exit(0);
  }
}

/////////////////////////////////////////////////////////////////////
///// ascii io routines
/////////////////////////////////////////////////////////////////////
void write_asc(char *filename, double* Pos, double *Vel, SIMinfo siminfo, int timeStep)
{
    FILE *fpo;
    fpo=fopen(filename,"w");
    long long nparticles=siminfo.numBodies;
    fprintf(fpo,"# %llu  %.4e\n",nparticles, siminfo.time);  
          // first line of files has # of particles and simulation time
    int i;
    for(i=0;i<nparticles;i++){
      fprintf(fpo,"%12.15e %12.15e %12.15e %12.15e %12.15e %12.15e %12.15e\n",
         Pos[4*i+3],Pos[4*i+0],Pos[4*i+1],Pos[4*i+2],  
         Vel[4*i+0],Vel[4*i+1],Vel[4*i+2]);
    }
    fclose(fpo);
}
void read_asc(char *filename, double* Pos, double *Vel, int timeStep)
{
    FILE *fpi;
    char string[300];
    ckfile(filename); // check if file exists
    fpi = fopen(filename,"r");
    int i;
    double m,x,y,z,vx,vy,vz;
    long long nparticles;
    // we need to get numbers of particles!
    fgets(string,300,fpi);
    sscanf(string,"#%llu",&nparticles);
    for(i=0;i<nparticles;i++){
       fgets(string,300,fpi);
       sscanf(string,"%lf %lf %lf %lf %lf %lf %lf",&m,&x,&y,&z,&vx,&vy,&vz);
       Pos[4*i+0] = x;  Pos[4*i+1] = y;  Pos[4*i+2] = z;
       Vel[4*i+0] = vx; Vel[4*i+1] = vy; Vel[4*i+2] = vz;
       Pos[4*i+3] = m;
    }

}

// return the number of bodies 
// and simulation time
void info_asc(char *filename, SIMinfo *siminfo)
{
    FILE *fpi;
    char string[300];
    ckfile(filename); // check if file exists
    fpi = fopen(filename,"r");
    // int i;
    long long nparticles;
    // we need to get numbers of particles!
    fgets(string,300,fpi);
    double time;
    sscanf(string,"#%llu %lf",&nparticles,&time);
    siminfo->numBodies = nparticles;
    siminfo->time= time;
}
