
//#include "hdf5.h"
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include <string.h>
#include <assert.h>
#include<iostream>
#include "kepcart.h"
#include "nbody.h"

// create initial conditions file for an nbody simulation
// argv1 is a parameter file


using namespace std;


// 
int main(int argc, char *argv[])
{
   SIMinfo siminfo;
   sread("param.in",  &siminfo); // read in param file 
   double *hPos, *hVel;
   int numBodies = siminfo.numBodies;
   int numBodiesMassive = siminfo.numBodiesMassive;
   hPos = new double[numBodies*4];
   hVel = new double[numBodies*4];
   siminfo.time = 0.0;
   cout << "running mkdisk" << endl;
   
   double GM=1.0;
   int i=0; // central body
     hPos[4*i+0] = 0.0;
     hPos[4*i+1] = 0.0;
     hPos[4*i+2] = 0.0;
     hPos[4*i+3] = GM;
     hVel[4*i+0] = 0.0;
     hVel[4*i+1] = 0.0;
     hVel[4*i+2] = 0.0;

   PhaseState pstate;
   OrbitalElements orbel;
// load up massive planets
   for(i=1;i<siminfo.nplanets+1;i++){
     int j = i-1;
     orbel.a = siminfo.orbel[j].a;
     orbel.e = siminfo.orbel[j].e;
     orbel.i = siminfo.orbel[j].i;
     orbel.longnode = siminfo.orbel[j].longnode;
     orbel.argperi  = siminfo.orbel[j].argperi;
     orbel.meananom = siminfo.orbel[j].meananom;
     double mp = siminfo.mplanets[j];
     cartesian(GM+mp,orbel,&pstate); // get keplerian values
     hPos[4*i+0] = pstate.x;  hPos[4*i+1] = pstate.y;  hPos[4*i+2] = pstate.z;
     hVel[4*i+0] = pstate.xd; hVel[4*i+1] = pstate.yd; hVel[4*i+2] = pstate.zd;
     hPos[4*i+3] = mp; // planet mass 
     hVel[4*i+3] = 0.0;
     printf("planet %d xyz %.2e %.2e %.2e %.2e %.2e %.2e %.2e\n", 
        i,pstate.x, pstate.y,pstate.z, 
        pstate.xd, pstate.yd,pstate.zd, mp ); 
     printf("planet %d oe  %.2e %.2e %.2e %.2e %.2e %.2e\n", 
        i,orbel.a, orbel.e,orbel.i, 
        orbel.longnode, orbel.argperi,orbel.meananom); 
     keplerian(GM+mp, pstate, &orbel);
     printf("check oe  %.2e %.2e %.2e %.2e %.2e %.2e\n", 
        orbel.a, orbel.e,orbel.i, 
        orbel.longnode, orbel.argperi,orbel.meananom); 
   }
   printf("nplanets= %d\n",siminfo.nplanets);

//  sigma(r) =  A r^gamma-1
   double mdisk =  siminfo.m*(double)(numBodiesMassive-siminfo.nplanets-1);
   double A;
   if (siminfo.gamma != -1.0)
      A = (siminfo.gamma+1.0)/
               (pow(siminfo.amax,siminfo.gamma+1.0) 
              - pow(siminfo.amin,siminfo.gamma+1.0));
   else A = 1.0/log(siminfo.amax/siminfo.amin);
   A *= mdisk/(2.0*M_PI);
   // mean surface density at inner and outer radius
   double sigmin = A*pow(siminfo.amin,siminfo.gamma-1.0);
   double sigmax = A*pow(siminfo.amax,siminfo.gamma-1.0);
   // mean spacing at inner and outer radius
   double smin = sqrt(siminfo.m/sigmin);
   double smax = sqrt(siminfo.m/sigmax);
   cout << "spacing inner " << smin << " s2 " << smin*smin << endl; 
   cout << "spacing outer " << smax << endl; 
   cout << "density inner (mass/area)" << sigmin << endl; 
   cout << "density outer " << sigmax << endl; 
   cout << "mdisk " << mdisk << endl; 
   double Q = siminfo.ibar/(M_PI*sigmin);
   cout << "Qin " << Q << endl; 

// smaller masses
   for(i=siminfo.nplanets+1;i<numBodies;i++){
    // powerlaw semimajor distn
     orbel.a = powerlaw(siminfo.amin,siminfo.amax,siminfo.gamma);
//  should adjust gamma by (1) r for circumference
//   sigma(r) =  A r^gamma-1
// generate random angles
     orbel.meananom= 2*M_PI*(double)rand()/(double)RAND_MAX; 
     orbel.argperi = 2*M_PI*(double)rand()/(double)RAND_MAX;
     orbel.longnode= 2*M_PI*(double)rand()/(double)RAND_MAX;
// generate random e and i using Rayleigh distribution
     orbel.e = rayleigh(2.0*siminfo.ibar); 
     orbel.i = rayleigh(siminfo.ibar);
     cartesian(GM,orbel,&pstate); // get keplerian values
     hPos[4*i+0] = pstate.x;  hPos[4*i+1] = pstate.y;  hPos[4*i+2] = pstate.z;
     hVel[4*i+0] = pstate.xd; hVel[4*i+1] = pstate.yd; hVel[4*i+2] = pstate.zd;
     if (i > numBodiesMassive)
       hPos[4*i+3] = 0.0; // massless particle       
     else
        hPos[4*i+3] = siminfo.m; //non planet or central particle mass
     hVel[4*i+3] = 0.0;
   }

   char filename[80];
   strcpy(filename,siminfo.outroot);

#ifdef H5PART
   strcat(filename,".h5part");
   cout << filename << endl;
   write_h5part(filename, hPos, hVel, siminfo, 0);
#else
   strcat(filename,".asc");
   cout << filename << endl;
   write_asc(filename, hPos, hVel, siminfo, 0);
#endif


   delete [] hPos;
   delete [] hVel;

}

