

// for all particles, return the particle id for the particle 
// that is closest 

 
#ifndef _NN_KERNEL_H_
#define _NN_KERNEL_H_

// #include <host_defines.h>

#define LOOP_UNROLL_N 4

struct iddist { float dist; int id; };

// convert a double4 into a float4
__device__ float4 d4tof4a(double4 dpos)
{
   float4 fpos;
   fpos.x = (float)dpos.x;
   fpos.y = (float)dpos.y;
   fpos.z = (float)dpos.z;
   fpos.w = (float)dpos.w;
   return fpos;
}

///////////////////////////////////////////////////////////
// ig is a distance id pair (struct iddist)
// if partible bj is closer to bi than the distance2 in ig.dist, 
// then update the ig.id and ig.dist and return the ig
// bi,bj are positions of particle b i,j
// we are putting the particle id into the mass slot for float bj
__device__ iddist  
bodyBodyInteraction(iddist ig, float4 bi, float4 bj) 
{
    float3 dr;
    iddist myig;
    myig = ig;

    dr.x = bj.x - bi.x;  
    dr.y = bj.y - bi.y;  
    dr.z = bj.z - bi.z; 
    double distSqr = dr.x*dr.x + dr.y*dr.y + dr.z*dr.z;
    if (distSqr == 0.0)   // self catch
       return myig;
    if (distSqr < ig.dist){
       myig.dist = distSqr;
       myig.dist = __float_as_int(bj.w);
       return myig;
    }
    return myig;
        
}


/////////////////////////////////////////////////////////
// loop through all particles through the blockdim
// This is a "tile_calculation" function 
/////////////////////////////////////////////////////////
__device__ iddist tile_calc(float4 myPos, iddist ig)
{
    extern __shared__ float4 sharedPosf[];
    int i=0;

    for (i = 0; i < blockDim.x; ) 
    {
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
#if LOOP_UNROLL_N > 1
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
#endif
#if LOOP_UNROLL_N > 2  
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
#endif
#if LOOP_UNROLL_N > 4
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
        ig  = bodyBodyInteraction(ig,myPos,sharedPosf[i]); i++;
#endif
    }

    return ig;
}

// WRAP is used to force each block to start working on a different 
// chunk (and wrap around back to the beginning of the array) so that
// not all multiprocessors try to read the same memory locations at 
// once.
#define WRAP(x,m) (((x)<m)?(x):(x-m))  // Mod without divide, works on values from 0 up to 2m

#define BIG 1e10
////////////////////////////////////////////////////////////////
// for every particle return the index of the closest particle
// Inputs are positions, numBodies
// Output is cindex array containing the index
////////////////////////////////////////////////////////////////

__global__ void  
findnearest(double4* Pos, int numBodies, int* cindex)
{
    extern __shared__ float4 sharedPosf[];
    int index = blockIdx.x*blockDim.x + threadIdx.x;
    float4 myPos = d4tof4a(Pos[index]);

    iddist ig;
    ig.dist = BIG; // initialize
    ig.id = -1; // 
    
    int p = blockDim.x;
    int n = numBodies;  // note numBodies could be num massive here
                        // and shorter than the total, as long as modp=0
    int i,tile;
   // number of tiles is N/p = blockdim.x which is why wrap works

    __syncthreads();

// load up mem for p particles then find min 
// the memory is transfered over and over again 
// but arranged so there is no interference
// between memory transfers

    for (i = 0, tile = 0; i < n; i += p, tile++)  // looping with step of tile (p) 
    { // load up shared memory 
        int idx = WRAP(blockIdx.x+tile, gridDim.x) * blockDim.x + threadIdx.x;
        sharedPosf[threadIdx.x].x = (float)Pos[idx].x;
        sharedPosf[threadIdx.x].y = (float)Pos[idx].y;
        sharedPosf[threadIdx.x].z = (float)Pos[idx].z;
        sharedPosf[threadIdx.x].w = __int_as_float(idx);
       
        __syncthreads();
        ig = tile_calc(myPos, ig); // get min disk and index p particles
        __syncthreads();
    }
    cindex[index] = ig.id;

}

#endif 

