
//#include "hdf5.h"
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include <string.h>
#include <assert.h>
#include<iostream>
#include "kepcart.h"
#include "nbody.h"

// take a h5part or asc file and spit out an ascii version listing orbital elements
//     printf("xxx.out datafilename \n");
// output is a .oe file

using namespace std;
void keplerian(double GM, PhaseState state, OrbitalElements *orbel);

// 
int main(int argc, char *argv[])
{
   SIMinfo siminfo;

   char filename[100];
   char ofilename[100];
   if (argc<2){
     printf("xxx.out h5partfilename \n");
     exit(0);
   }
   strcpy(filename,argv[1]);
#ifdef H5PART
   info_h5part(filename, &siminfo); // get number of bodies
#else        
   info_asc(filename, &siminfo); // get number of bodies and time
#endif       
   int timestep=0;
//   sscanf(argv[2],"%d",&timestep);
//   printf("hello\n");

   double *hPos, *hVel;
   int numBodies = siminfo.numBodies;
   hPos = new double[numBodies*4];
   hVel = new double[numBodies*4];
//  printf("numBodies %d time=%.3e\n",numBodies,siminfo.time);
#ifdef H5PART
   read_h5part(filename, hPos, hVel,  timestep); // get data
#else        
   read_asc(filename, hPos, hVel,  timestep);
#endif       

   FILE *fpo;
   strcpy(ofilename,filename);
   int pch = strcspn(ofilename,".");
   ofilename[pch]='\0';
   strcat(ofilename,".oe");
   fpo=fopen(ofilename,"w");
   fprintf(fpo,"# m a e i longnode argperi meananom                P=%.3f\n"
       ,siminfo.time/(2.0*M_PI));
   int i;
   // central star
   double x0  = hPos[4*0+0]; double y0  = hPos[4*0+1]; double z0  = hPos[4*0+2]; 
   double vx0 = hVel[4*0+0]; double vy0 = hVel[4*0+1]; double vz0 = hVel[4*0+2]; 
   double GM = hPos[4*0+3];
   PhaseState pstate;
   OrbitalElements orbel;
   for(i=1;i<numBodies;i++){
     pstate.x  = hPos[4*i+0]-x0;  pstate.y  = hPos[4*i+1]-y0;  
     pstate.xd = hVel[4*i+0]-vx0; pstate.yd = hVel[4*i+1]-vy0; 
     pstate.z  = hPos[4*i+2]-z0;
     pstate.zd = hVel[4*i+2]-vz0;
//     if (i<5) printf("%.2e %.2e %.2e %.2e %.2e %.2e \n"
//        ,pstate.x, pstate.y, pstate.z, 
//        pstate.xd, pstate.yd, pstate.zd); 
     double m = hPos[4*i+3];
     keplerian(GM+m, pstate, &orbel);
     fprintf(fpo,"%11.3e %11.3e %11.3e %11.3e %11.3e %11.3e %11.3e\n",
         m, orbel.a, orbel.e, orbel.i, 
         orbel.longnode, orbel.argperi, orbel.meananom);
   }
   fclose(fpo);

   delete [] hPos;
   delete [] hVel;

}

